var objControl;
var objTree;
var objRoot;

var isMac = /Mac/.test(navigator.platform);
var InUserEdit = false;

// Mouse Event Support
var IsMouseDown = false;
var MouseButton = 1;

function GetX(event)
{
	var x = event.pageX;
	var intMargin = parseInt($('#wccControl').css('margin-left'));
	var intScroll = $('#wccContainer').scrollLeft();
	return (x - intMargin) + intScroll;
}

function GetY(event)
{
	var y = event.pageY;
	var intMargin = parseInt($('#wccControl').css('margin-top'));
	var intScroll = $('#wccContainer').scrollTop();
	return (y - intMargin) + intScroll;
}

// Content resize check.
// Rather than try to hook all the actions which can lead to a content
// resize we just check it every so often and fire an event if it has
// changed. It's not the best solution, but until mutation events are
// standard across browsers, it's the best way to insure that size
// changes are always reported to RB.
//
// Note that frequency is limited to 200ms to try and conserve CPU time on
// the browser. Not that this would be huge, but there's little point in checking
// at a faster rate.
var intLastWidth = -1;
var intLastHeight = -1;

function CheckContentSize()
{
	var objSize = wcc.GetContentSize();
	
	if (objSize.width != intLastWidth || objSize.height != intLastHeight)
	{
		intLastWidth = objSize.width;
		intLastHeight = objSize.height;
		wcc.RaiseEvent(wcc.kSingle, "wccContentResize", {"Width":objSize.width, "Height":objSize.height});
	}
	
	// Test again later.
	var intTime = Math.max(200, wcc.EventFrequency);
	setTimeout('CheckContentSize();', intTime);
}

function StartupControl(objOptions)
{
	// Bind event handlers.
	objOptions.onClick = NodeClick;
	objOptions.onDblClick = NodeDoubleClick;
	objOptions.onKeydown = NodeKeypress;
	
	objOptions.onSelect = OnSelect;
	objOptions.onExpand = OnExpand;
	objOptions.onActivate = OnActivate;
	objOptions.onDeactivate = OnDeactivate;
	objOptions.onFocus = OnFocus;
	objOptions.onBlur = OnBlur;
	
	objOptions.dnd.onDragStart = NodeDragStart;
	objOptions.dnd.onDragEnter = NodeDragEnter;
	objOptions.dnd.onDrop = NodeDrop;
	
	//Startup the control.
	objControl = $('#wccControl');
	objControl.dynatree(objOptions);
	
	// Setup the context.
	objTree = objControl.dynatree('getTree');
	objRoot = objControl.dynatree('getRoot');
	
	// Events
	CheckContentSize();	
	
	// Raising clicks on the outer control allows us
	// to deactivate the active node if desired.
	$('#wccContainer').bind('click', function (event) 
    {
        if (event.target.id === "wccContainer")
        	wcc.RaiseEvent(wcc.kMulti, "wccClick", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
}

// Event handlers.
function NodeDragStart(sourceNode)
{
	var bolControl = objControl.dynatree('option', 'wccNodeDragAndDrop');
	var bolNode = sourceNode.data.wccAllowDrag;
	return (bolControl && bolNode);
}

function NodeDragEnter(targetNode, sourceNode)
{
	// Does the targetNode accept drops?
	if (targetNode.data.wccAllowDrop == true) 
		return true;
	else
		return ['before', 'after'];
}

function NodeDrop(targetNode, sourceNode, hitMode, ui, draggable)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeDrop", {strSourceKey:sourceNode.data.key, strTargetKey:targetNode.data.key, strHitMode:hitMode});
}

function NodeKeypress(dtnode, event)
{
	// Notify wcc.
	var objKeypress = wcc.GetKeyEvent(event);
	wcc.RaiseEvent(wcc.kMulti, "wccNodeKeypress", {strNodeKey:dtnode.data.key, objEvent:objKeypress});
	return true;
}

function NodeClick(dtnode, event)
{
	// Notify wcc.
	wcc.RaiseEvent(wcc.kMulti, "wccNodeClick", {strNodeKey:dtnode.data.key});
	return true;
}

function NodeDoubleClick(dtnode, event)
{
	// Notify wcc.
	wcc.RaiseEvent(wcc.kMulti, "wccNodeDoubleClick", {strNodeKey:dtnode.data.key});
	return true;
}

function OnFocus(dtnode, event)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeFocus", {strNodeKey:dtnode.data.key, bolFlag:true});
}

function OnBlur(dtnode, event)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeFocus", {strNodeKey:dtnode.data.key, bolFlag:false});
}

function OnActivate(dtnode)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeActivate", {strNodeKey:dtnode.data.key, bolFlag:true});
}

function OnDeactivate(dtnode)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeActivate", {strNodeKey:dtnode.data.key, bolFlag:false});
}

function OnSelect(flag, dtnode)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeSelect", {strNodeKey:dtnode.data.key, bolFlag:flag});
}

function OnExpand(flag, dtnode)
{
	wcc.RaiseEvent(wcc.kNow, "wccNodeExpand", {strNodeKey:dtnode.data.key, bolFlag:flag});
}

// Node move in response to drop.
function MoveNode(strSourceKey, strTargetKey, strHitMode)
{
	debugger;
	var objSource = FindNode(strSourceKey);
	if (objSource == null) return;
	
	var objTarget = FindNode(strTargetKey);
	if (objTarget == null) return;
	
	objSource.move(objTarget, strHitMode);
}

// Control functions.
function SetPreventVoidDrops(bolValue)
{
	var dnd = objControl.dynatree('option', 'dnd');
	dnd.preventVoidMoves = bolValue;
}

function SetDropAutoExpand(intValue)
{
	var dnd = objControl.dynatree('option', 'dnd');
	dnd.autoExpandMS = intValue;
}

// Node functions.
function FindNode(strKey)
{
	return objControl.dynatree("getTree").getNodeByKey(strKey);
}

function InsertNode(strParentKey, objChild, strBeforeKey)
{
	// Get nodes.
	var objParent = FindNode(strParentKey);
	if (objParent == null) return;
	
	var objBefore = null;
	if (wcc.RealTypeOf(strBeforeKey) == 'String')
		objBefore = FindNode(strBeforeKey);
	
	// Add node.
	if (objBefore == null)
		objParent.addChild(objChild);
	else
		objParent.addChild(objChild, objBefore);
}

function RemoveNode(strKey)
{
	var objNode = FindNode(strKey);
	if (objNode != null)
	{
		// We have to render the node or else it will not be removed.
		objNode.render();
		objNode.remove();
	}
}

function ActivateNode(strKey, bolActive, bolSilent)
{
	var objNode = FindNode(strKey);
	if (objNode != null)
	{
		if (bolActive == true)
		{
			if (bolSilent == true)
				objNode.activateSilently();
			else
				objNode.activate();
		}
		else
			objNode.deactivate();
	}
}

function ScrollToNode(strKey, intScrollSpeed)
{
	// Scroll into view.
	$('#wccContainer').animate
	(
		{scrollTop: $('#wccContainer').scrollTop() + $('#dynatree-id-' + strKey).offset().top}, 
		intScrollSpeed
	);
}

function ClearPartialSelect()
{
	setTimeout("$('.dynatree-partsel').removeClass('dynatree-partsel');", 500);
}

function EditTitleInline(strKey)
{
	// Get the node.
	var node = FindNode(strKey);
	if (node == null) return;
	
	// Notify wcc.
	wcc.RaiseEvent(wcc.kNow, "wccInUserEdit", {strNodeKey:node.data.key, bolFlag:true});
	
	// Edit the node.
	var prevTitle = node.data.title,
	tree = node.tree;
	
	// Disable dynatree mouse- and key handling
	tree.$widget.unbind();
	
	// Replace node with <input>
	$(".dynatree-title", node.span).html("<input id='editNode' value='" + prevTitle + "'>");
	
	// Focus <input> and bind keyboard handler
	$("input#editNode")
	.focus()
	.keydown(function(event){
	switch( event.which ) {
		case 27: // [esc]
		// discard changes on [esc]
		$("input#editNode").val(prevTitle);
		$(this).blur();
		break;
		case 13: // [enter]
		// simulate blur to accept new value
		$(this).blur();
		break;
	}
	}).blur(function(event){
		// Accept new value, when user leaves <input>
		var title = $("input#editNode").val();
		node.setTitle(title);
		// Re-enable mouse and keyboard handlling
		tree.$widget.bind();
		node.focus();
		// Notify wcc.
		wcc.RaiseEvent(wcc.kMulti, "wccInUserEdit", {strNodeKey:node.data.key, bolFlag:false});
		wcc.RaiseEvent(wcc.kMulti, "wccUserEditNode", {strNodeKey:node.data.key, strOldTitle:prevTitle, strNewTitle:title});
	});
}