// wcc.jcanvas.js
// Version: 1.3.1
// By Daniel L. Taylor
// Copyright (C) 2011-2013 Taylor Design. All Rights Reserved.
// http://www.taylor-design.com/
// **********************************************************************

var objCanvas;
var objContext;

function StartupControl()
{
	// Store the control.
	objCanvas = $('#wccControl');
	objContext = objCanvas.get(0).getContext('2d');
	
	// Store wcc reference. This is required for some callbacks.
	objCanvas.get(0).wcc = wcc;
	
	// fromCenter should not default to true. It's just not the way most
	// people think about graphic layout, though it is useful for some things.
	// AutoSave interferes with the RB class representation. Set to false.
	$.jCanvas({fromCenter: false, cropFromCenter: false, autosave:false});
	
	// Just makes it easy to use the event code below, copied from another control.
	var obj = objCanvas;
	
	// Bind events.
    // click
	obj.bind('click', function (event) 
    {
        wcc.RaiseEvent(wcc.kMulti, "wccClick", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    }); 
    
    // dblclick
	obj.bind('dblclick', function (event) 
    {
        wcc.RaiseEvent(wcc.kMulti, "wccDoubleClick", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
    
    // mouseover
    obj.bind('mouseover', function (event) 
    {
        // Are we already in?
        if (InCanvas == true) return;
        
		// We're in.
		InCanvas = true;
        IsMouseDown = false;
        MouseButton = 1;
        wcc.RaiseEvent(wcc.kClear, "wccMouseEnter");
    });
    
    // mouseout
    obj.bind('mouseout', function (event) 
    {
        // Get Canvas element.
		var obj = document.getElementById('wccControl');
		if (obj == null || obj == undefined)
			return;
			
		// Due to a bug, we need to test the mouse position and
		// ignore if it's still inside the canvas.
		var x = event.pageX;
		var y = event.pageY;
		if ((x < obj.width-2 && x > 2) && (y < obj.height-2 && y > 2)) return;
        
        // We're out.
        InCanvas = false;
        IsMouseDown = false;
        MouseButton = 1;
        LayersOut();
        wcc.RaiseEvent(wcc.kClear, "wccMouseExit");
    });
    
    // mousemove
	obj.bind('mousemove', function (event) 
	{
		InCanvas = true;
	    if (IsMouseDown == true)
	    	wcc.RaiseEvent(wcc.kSingle, "wccMouseDrag", {"x":GetX(event),"y":GetY(event),"Button":MouseButton} );
	    else
	    	wcc.RaiseEvent(wcc.kSingle, "wccMouseMove", {"x":GetX(event),"y":GetY(event)} );
	});
    
    // mousedown
	obj.bind('mousedown', function (event) 
    {
        IsMouseDown = true;
        MouseButton = event.which;
        wcc.RaiseEvent(wcc.kClear, "wccMouseDown", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
    
    // mouseup
	obj.bind('mouseup', function (event) 
    {
        IsMouseDown = false;
        MouseButton = event.which;
        wcc.RaiseEvent(wcc.kClear, "wccMouseUp", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
	
	// Control loaded.
	wcc.ControlLoaded();
}

// Layer events
function LayersOut()
{
	var objLayers = objCanvas.getLayers();
	var c = objLayers.length - 1;
	for (var i = c; i >= 0; i--)
	{
		if (objLayers[i].IsMouseOver == true)
		{
			wcc.RaiseEvent(wcc.kMulti, "wccLayerMouseExit", {"id":objLayers[i].name} );
			objLayers[i].IsMouseOver = false;
		}
	}
}

function LayerClick(layer) 
{
	var x = layer.mouseX - layer.x;
	var y = layer.mouseY - layer.y;
	wcc.RaiseEvent(wcc.kSingle, "wccLayerClick", {"id":layer.name, "x":x, "y":y, "Button":MouseButton} );
}

function LayerDoubleClick(layer)
{
	var x = layer.mouseX - layer.x;
	var y = layer.mouseY - layer.y;
	wcc.RaiseEvent(wcc.kSingle, "wccLayerDoubleClick", {"id":layer.name, "x":x, "y":y, "Button":MouseButton} );
}

function LayerMouseDown(layer) 
{
	var x = layer.mouseX - layer.x;
	var y = layer.mouseY - layer.y;
	wcc.RaiseEvent(wcc.kSingle, "wccLayerMouseDown", {"id":layer.name, "x":x, "y":y, "Button":MouseButton} );
}

function LayerMouseUp(layer) 
{
	var x = layer.mouseX - layer.x;
	var y = layer.mouseY - layer.y;
	wcc.RaiseEvent(wcc.kSingle, "wccLayerMouseUp", {"id":layer.name, "x":x, "y":y, "Button":MouseButton} );
}

function LayerMouseEnter(layer)
{
	// Mark the layer.
	layer.IsMouseOver = true;
	
	// Variables
	var objLayers = objCanvas.getLayers();
	var intIndex = objLayers.indexOf(layer);
	var c = objLayers.length - 1;
	var i;
	
	// Is there a layer above this one?
	for (i = c; i > intIndex; i--)
	{
		if (objLayers[i].IsMouseOver == true)
			return;
	}
    
    // Is there a layer below?
    for (i = (intIndex - 1); i >= 0; i--)
    {
        if (objLayers[i].IsMouseOver == true)
        {
            wcc.RaiseEvent(wcc.kMulti, "wccLayerMouseExit", {"id":objLayers[i].name} );
            i = -1; // Only call for the first found.
        }
    }
    
    // Send event.
    wcc.RaiseEvent(wcc.kMulti, "wccLayerMouseEnter", {"id":layer.name} );
}

function LayerMouseExit(layer)
{
	// Is this a boundry situation where we already called Exit
	// in response to a control exit?
	if (layer.IsMouseOver == false) return;
	
	// Mark the layer.
	layer.IsMouseOver = false;
	
	// Variables
	var objLayers = objCanvas.getLayers();
	var intIndex = objLayers.indexOf(layer);
	var c = objLayers.length - 1;
	var i;
	
	// Is there a layer above this one?
	for (i = c; i > intIndex; i--)
	{
		if (objLayers[i].IsMouseOver == true)
			return;
	}
	
	// Send event.
    wcc.RaiseEvent(wcc.kMulti, "wccLayerMouseExit", {"id":layer.name} );
    
    // Is there a layer below?
    for (i = (intIndex - 1); i >= 0; i--)
    {
        if (objLayers[i].IsMouseOver == true)
        {
            wcc.RaiseEvent(wcc.kMulti, "wccLayerMouseEnter", {"id":objLayers[i].name} );
            return;
        }
    }
}

// Canvas resize
function SetCanvasSize(intWidth, intHeight)
{
	// Preflight
	if (wcc.RealTypeOf(intWidth) != 'Number' || wcc.RealTypeOf(intHeight) != 'Number')
		return;
		
	// Get Canvas element.
	var obj = document.getElementById('wccControl');
	if (obj == null || obj == undefined)
		return;
		
	// This is how we have to set the size with a Canvas object.
	// Setting it via CSS distorts the rendered area.
	obj.width = intWidth;
	obj.height = intHeight;
	
	// Raise event.
	wcc.RaiseEvent(wcc.kSingle, "wccContentResize", {"Width":intWidth, "Height":intHeight});
}

function GetCanvasImage(strFormat, dblQuality)
{
	// Preflight
	if (strFormat != 'jpeg' && strFormat != 'png')
		strFormat = 'png';
	
	if (wcc.RealTypeOf(dblQuality) != 'Number')
		dblQuality = 0.7;
	
	var s = objCanvas.getCanvasImage(strFormat, dblQuality);
	wcc.RaiseEvent(wcc.kSingle, "wccCanvasImage", {"Value":s});
}

// Mouse Event Support
var InCanvas = false;
var IsMouseDown = false;
var MouseButton = 1;

function GetX(event)
{
	var x = event.pageX;
	var intMargin = parseInt($('#wccControl').css('margin-left'));
	var intScroll = $('#wccContainer').scrollLeft();
	return (x - intMargin) + intScroll;
}

function GetY(event)
{
	var y = event.pageY;
	var intMargin = parseInt($('#wccControl').css('margin-top'));
	var intScroll = $('#wccContainer').scrollTop();
	return (y - intMargin) + intScroll;
}

// ZIndex Support
function SortLayers()
{
	var objLayers = objCanvas.getLayers();
	objLayers.sort(function (a,b) 
	{
	  if (a.ZIndex < b.ZIndex)
	     return -1;
	  if (a.ZIndex > b.ZIndex)
	    return 1;
	  return 0;
	});
}

// Font Utilities
function LoadFont(strUrl, strFont, strFlag)
{
	WebFont.load({	custom: { families: [strFont],
					urls: [strUrl] },
					fontactive: function (fontFamily, fontDescription) 
					{
    					wcc.RaiseEvent(wcc.kMulti, "wccFontLoaded", {"Flag":strFlag, "Font":strFont} );
  					},
  					fontinactive: function (fontFamily, fontDescription) 
  					{
					    wcc.RaiseEvent(wcc.kMulti, "wccFontLoadFailure", {"Flag":strFlag, "Font":strFont} );
					},
				});
}
function MeasureStringWidth(strFont, strText)
{
	// Swap fonts.
	var strOldFont = objContext.font;
	objContext.font = strFont;
	
	// Get the data.
	var objMetrics = objContext.measureText(strText);
	
	// Clean up.
	objContext.font = strOldFont;
	return objMetrics.width;
}

function GetStringWidth(strFont, strFlag, strText)
{
	numWidth = MeasureStringWidth(strFont, strText);
	wcc.RaiseEvent(wcc.kMulti, "wccStringWidth", {"Flag":strFlag, "Width":numWidth} );
}

function GetStringHeight(strFont, strFlag, strText, numWrapWidth) 
{
	var objText = $('<div style="font: ' + strFont + '; min-width:' + numWrapWidth + 'px; width:' + numWrapWidth +'px;">' + strText + '</div>');
	var objBody = $('body');
	objBody.append(objText);
	var numHeight = objText.height();
	objText.remove();
	wcc.RaiseEvent(wcc.kMulti, "wccStringHeight", {"Flag":strFlag, "Height":numHeight} );
};

function LoadAllFontMetrics(strFlag, strFonts)
{
	var objList = new Array;
	var c = strFonts.length;
	for (var i = 0; i < c; i++)
	{
		var objMetrics = GetFontMetrics(strFonts[i]);
		objList.push(objMetrics);
	}
	
	wcc.RaiseEvent(wcc.kMulti, "wccAllFontMetricsLoaded", {"Value":objList, "Flag":strFlag} );
}

function LoadFontMetrics(strFont)
{
	var objResult = GetFontMetrics(strFont);
	wcc.RaiseEvent(wcc.kMulti, "wccFontMetricsLoaded", {"Value":objResult} );
}

function GetFontMetrics(strFont) 
{
	// Build the test elements.
	var objText = $('<span style="font: ' + strFont + '">Hg</span>');
	var objBlock = $('<div style="display: inline-block; width: 1px; height: 0px;"></div>');
	
	var objDiv = $('<div></div>');
	objDiv.append(objText, objBlock);
	
	var objBody = $('body');
	objBody.append(objDiv);
	
	// Compute the metrics.
	var objResult = {};
	try 
	{
		objBlock.css({ verticalAlign: 'baseline' });
		objResult.ascent = objBlock.offset().top - objText.offset().top;
		
		objBlock.css({ verticalAlign: 'bottom' });
		objResult.height = objBlock.offset().top - objText.offset().top;
		
		objResult.descent = objResult.height - objResult.ascent;
	} 
	finally 
	{
		objDiv.remove();
	}
	
	// Set the key and return.
	objResult.key = strFont;
	return objResult;
}

function OnCompletion(strFlag)
{
	wcc.RaiseEvent(wcc.kNow, "wccCompleted", {"Flag":strFlag} );
}

// Frame Animation ********************************************

// Shim layer with setTimeout fallback.
window.requestAnimFrame = (function(){
  return  window.requestAnimationFrame       || 
          window.webkitRequestAnimationFrame || 
          window.mozRequestAnimationFrame    || 
          window.oRequestAnimationFrame      || 
          window.msRequestAnimationFrame     || 
          function( callback ){
            window.setTimeout(callback, 1000 / 60);
          };
})();

// Variables
var objFrames = new Array;
var numCycle = 1;
var numMinQueueDepth = -1;
var bolFramesRequested = false;
var idProcessor = null;

// Statistics & Drifting
var bolDriftCompensation = true;
var bolDriftDropFrames = true;
var numDriftDropThreshold = 30;
var numDriftStartTime = 0;
var numFrameSlice = 1000 / 60;
var numFramesDoubled = 0;

var numTimerEvents = 0;
var numFramesDrawn = 0;
var numFramesDropped = 0;

var numDrawTotalTime = 0;
var numDrawMaxTime = 0;

// Max Frame Rate Computation
var numMaxRateStart;
var numMaxRateCount;

// Functions called from WE.
function GetFrameStatistics(strFlag)
{
	var jsnStats = new Object;
	jsnStats.Flag = strFlag;
	
	jsnStats.Time = new Date;
	jsnStats.QueueDepth = objFrames.length;
	jsnStats.FrameTime = numFrameSlice;
	jsnStats.FramesDoubled = numFramesDoubled;
	jsnStats.FramesDrawn = numFramesDrawn;
	jsnStats.FramesDropped = numFramesDropped;
	jsnStats.TimerEvents = numTimerEvents;
	jsnStats.AverageDrawTime = numDrawTotalTime / numFramesDrawn;
	jsnStats.MaxDrawTime = numDrawMaxTime;
	
	wcc.RaiseEvent(wcc.kNow, "wccFrameStatistics", {"Stats":jsnStats});
}

function GetMaxFrameRateStart()
{
	numMaxRateStart = (new Date).valueOf();
	numMaxRateCount = 0;
	requestAnimFrame(GetMaxFrameRateFinish);
}

function GetMaxFrameRateFinish()
{	
	var numNow = (new Date).valueOf();
	if ((numNow - numMaxRateStart) >= 1000)
	{
		numFrameSlice = 1000 / numMaxRateCount;
		wcc.RaiseEvent(wcc.kNow, "wccMaxFrameRate", {"FrameRate":numMaxRateCount});
	}
	else
	{
		numMaxRateCount++;
		requestAnimFrame(GetMaxFrameRateFinish);
	}
}

function QueueFrame(objFrame)
{
	bolFramesRequested = false;
	objFrames.push(objFrame);
	if (objFrames.length == 1)
	{
		numCycle = 1;
		ResetStatistics();
	}
		
	if (idProcessor == null)
		ScheduleNextFrame();
}

function ClearFrameQueue(bolProcessKeyFrames)
{
	// Key frames?
	if (bolProcessKeyFrames == true)
	{
		for (objFrame in objFrames)
		{
			// We do not add to the stats here because it would severely distort the
			// information that the user is looking for.
			if (objFrame.IsKey == true)
				objFrame.Draw();
		}
	}
	
	// Clear the queue.
	objFrames = new Array;
	numCycle = 1;
	bolFramesRequested = false;
	ResetStatistics();
}

// Supporting functions.
function ResetStatistics()
{
	numDriftStartTime = (new Date).valueOf();
	numFramesDoubled = 0;
	
	numTimerEvents = 0;
	numFramesDrawn = 0;
	numFramesDropped = 0;
	
	numDrawTotalTime = 0;
	numDrawMaxTime = 0;
}

function ScheduleNextFrame()
{
	if (idProcessor != null) return;
	if (objFrames.length < 1) return;
	idProcessor = requestAnimFrame(ProcessFrame);
}

function ProcessFrame()
{
	try
	{
		// Clear ID and mark.
		idProcessor = null;
		numTimerEvents++;
		
		// Any frames? If not, we don't need to keep processing.
		if (objFrames.length < 1)
		{
			RequestFrames();
			return;
		}
		
		// Schedule. This should happen here even though
		// the next section might empty the queue. This is
		// because the fallback for the shim is setTimeout
		// and we don't want that to drift due to later
		// processing.
		ScheduleNextFrame();
		
		// Draw the next frame.
		DrawFrame();
		numFramesDrawn++;
			
		// Drift Compensation
		var numNow = (new Date).valueOf();
		var numElapsed = numNow - numDriftStartTime;
		var numExpectedFrames = numElapsed / numFrameSlice;
		var numFrameDiff = numExpectedFrames - (numFramesDrawn + numFramesDropped);
		
		if (bolDriftCompensation == true && numFrameDiff > 0)
		{
			if (bolDriftDropFrames == true && numFrameDiff > numDriftDropThreshold)
			{
				// Drop non-key frames. Draw but do not count key frames. Leave the last frame.
				var c = Math.min(objFrames.length - 1, numFrameDiff);
				if (c > 0)
				{
					var objDropList = objFrames.splice(0,c);
					for (objFrame in objDropList)
					{
						if (objFrame.IsKey == true) objFrame.Draw();
						numFramesDropped++;
					}
				}
			}
			else
			{
				DrawFrame();
				numFramesDrawn++;
				numFramesDoubled++;
			}
		}
		
		// Request more.
		RequestFrames();
	}
	catch (err)
	{
		// Do nothing for now.
	}
}

function DrawFrame()
{
	try
	{
		if (objFrames.length < 1) return;
		var objFrame = objFrames[0];
		if (objFrame.Cycle == numCycle)
		{
			// Draw the frame.
			var ts = (new Date).valueOf();
			objFrame.Draw();
			var t = (new Date).valueOf() - ts;
			numDrawTotalTime += t;
			if (numDrawMaxTime < t) numDrawMaxTime = t;
			
			// Adjust the queue.
			objFrames.splice(0,1);
			numCycle = 1;
		}
		else
			numCycle++;
	}
	catch (err)
	{
		// Do nothing for now.
	}
}

function RequestFrames()
{
	// Are there enough frames for now?
	if (objFrames.length > numMinQueueDepth) return;
	
	// Has a request already been sent?
	if (bolFramesRequested == true) return;
	bolFramesRequested = true;
	
	// Raise event.
	wcc.RaiseEvent(wcc.kNow, "wccFrameRequest", {"FramesLeft":objFrames.length} );
}