// wcc.jquerybutton.js
// Version: 1.2
// By Daniel L. Taylor
// Copyright (C) 2011-2012 Taylor Design. All Rights Reserved.
// http://www.taylor-design.com/
// **********************************************************************

// For some stupid reason jQueryButton will break the caption when aligned right.
// It's as if offsetWidth returns a value which is a couple pixels too short.
// So we override the related wcc function here.
wcc.SetAlignHorizontal = function (intAlignHorizontal)
{
	// Store value.
	wcc.AlignHorizontal = intAlignHorizontal;
	
	// Get divs.
	var objContainer = wcc.GetContainerDiv();
	if (objContainer === null || objContainer === undefined)
		return;
	
	var objControl = wcc.GetControlDiv();
	if (objControl === null || objControl === undefined)
		return;
	
	// Compute layout and adjust.
	var x = 0;
	
	switch(wcc.AlignHorizontal)
	{
		case wcc.kAlignDefault:
			x = 0;
			break;
		case wcc.kAlignLeft:
			x = 0;
			break;
		case wcc.kAlignCenter:
			x = (objContainer.offsetWidth - objControl.offsetWidth) / 2;
			break;
		case wcc.kAlignRight: // Modification is here.
			x = objContainer.offsetWidth - (objControl.offsetWidth + 5);
			break;
	}

	objControl.style.marginLeft = x + 'px';
}

// Mouse Event Support
var IsMouseDown = false;
var MouseButton = 1;

function GetX(event)
{
	var x = event.pageX;
	var intMargin = parseInt($('#wccControl').css('margin-left'));
	var intScroll = $('#wccContainer').scrollLeft();
	return (x - intMargin) + intScroll;
}

function GetY(event)
{
	var y = event.pageY;
	var intMargin = parseInt($('#wccControl').css('margin-top'));
	var intScroll = $('#wccContainer').scrollTop();
	return (y - intMargin) + intScroll;
}

function StartupControl()
{
	// Get the jQuery object.
	var obj = $('#wccControl');
	
	// Disable default drag behavior.
	obj.bind('dragstart', function(event) { event.preventDefault(); });

	// Bind events.
    // click
	obj.bind('click', function (event) 
    {
        wcc.RaiseEvent(wcc.kMulti, "wccClick", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    }); 
    
    // dblclick
	obj.bind('dblclick', function (event) 
    {
        wcc.RaiseEvent(wcc.kMulti, "wccDoubleClick", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
    
    // mouseover
    obj.bind('mouseover', function (event) 
    {
        IsMouseDown = false;
        MouseButton = 1;
        wcc.RaiseEvent(wcc.kClear, "wccMouseEnter");
    });
    
    // mouseout
    obj.bind('mouseout', function (event) 
    {
        IsMouseDown = false;
        MouseButton = 1;
        wcc.RaiseEvent(wcc.kClear, "wccMouseExit");
    });
    
    // mousemove
	obj.bind('mousemove', function (event) 
    {
        if (IsMouseDown == true)
        	wcc.RaiseEvent(wcc.kSingle, "wccMouseDrag", {"x":GetX(event),"y":GetY(event),"Button":MouseButton} );
        else
        	wcc.RaiseEvent(wcc.kSingle, "wccMouseMove", {"x":GetX(event),"y":GetY(event)} );
    });
    
    // mousedown
	obj.bind('mousedown', function (event) 
    {
        IsMouseDown = true;
        MouseButton = event.which;
        wcc.RaiseEvent(wcc.kClear, "wccMouseDown", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
    
    // mouseup
	obj.bind('mouseup', function (event) 
    {
        IsMouseDown = false;
        MouseButton = event.which;
        wcc.RaiseEvent(wcc.kClear, "wccMouseUp", {"x":GetX(event),"y":GetY(event),"Button":event.which} );
    });
    
    // create
    obj.bind('buttoncreate', function (event, ui) 
    {
        if (wcc.AlignHorizontal > wcc.kAlignLeft || wcc.AlignVertical > wcc.kAlignTop)
        {
        	// Unfortunately there has to be a delay or the button will not size and align correctly.
	        setTimeout('FinishStartup();', 100);
        }
        else
        	FinishStartup();
    });
    
    // FastStart
    wcc.FastStart(objOptions);
    
    // Set jQueryBase options.
	wcc.SetControlFontSize(objOptions.FontSize);
	wcc.SetCssLinkById('theme', objOptions.ThemeUrl);
	
    // Startup
	obj.button(objOptions.jQueryStartup);
	// Note that startup is completed on the jQuery create event (above).
}

function FinishStartup()
{
	// Set control size. We have to do this again here
	// with jQueryButton.
	var strType = wcc.RealTypeOf(objOptions.ControlWidth);
	if (strType == 'Number' || strType == 'String')
	{
		wcc.SetControlSize(objOptions.ControlWidth, objOptions.ControlHeight);
	}
	
	// Align and make visible. Note that we have to realign
	// after a delay due to the way jQueryButton starts up.
	wcc.SetAlignHorizontal(wcc.AlignHorizontal);
	wcc.SetAlignVertical(wcc.AlignVertical);
	document.getElementById('wccControl').style.visibility = 'visible';
	
	// Notify RB.
	wcc.ControlLoaded();
}